using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.Win32;

using MediaPortal.GUI.Library;

namespace WindowPlugins.BluRayPlayerLauncher
{
  public static class InstalledPlayers
  {
    public static string GetPlayerExePath(Players player)
    {
      switch (player)
      {
        case Players.Arcsoft:
          return InstalledPlayers.GetArcsoftExePath();
        case Players.Cyberlink:
          return InstalledPlayers.GetCyberlinkExePath();
        default:
          return string.Empty;
      }
    }

    private static string GetArcsoftExePath()
    {
      string path = InstalledPlayers.GetTMT5ExePath();
      if (path != string.Empty)
      {
          return path;
      }
      else
      {
          return InstalledPlayers.GetTMT3ExePath();
      }
    }

    private static string GetCyberlinkExePath()
    {
      //currently PDVD9 isn't supported, because it doesn't react on the keystroke emulation like it is atm
      //string path = InstalledPlayers.GetPowerDVD9ExePath();
      //if (path != string.Empty)
      //  return path;

      return InstalledPlayers.GetPowerDVDExePath();
    }

    private static string GetTMT3ExePath()
    {
      try
      {
        RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\ArcSoft\TotalMedia Theatre 3\3.0\");
        if (key == null)
          return string.Empty;

        object value = key.GetValue("AppPath", null);
        if (value == null)
          return string.Empty;

        string path = value.ToString() + "uMCEDVDPlayer.exe";
        if (!System.IO.File.Exists(path))
          return string.Empty;

        return path;
      }
      catch (Exception exp)
      {
        Log.Error(exp);
        return string.Empty;
      }
    }

    private static string GetTMT5ExePath()
    {
        try
        {
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\ArcSoft\TotalMedia Theatre 5\5.0\");
            if (key == null)
                return string.Empty;

            object value = key.GetValue("AppPath", null);
            if (value == null)
                return string.Empty;

            string path = value.ToString() + "uMCEDVD5Player.exe";
            if (!System.IO.File.Exists(path))
                return string.Empty;

            return path;
        }
        catch (Exception exp)
        {
            Log.Error(exp);
            return string.Empty;
        }
    }

    private static string GetPowerDVDExePath()
    {
      try
      {
        RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\CyberLink\PowerDVD\");
        if (key == null)
          return string.Empty;

        object value = key.GetValue("InstallPath", null);
        if (value == null)
          return string.Empty;

        string path = value.ToString() + @"\PowerDVD.exe";
        if (!System.IO.File.Exists(path))
          return string.Empty;

        return path;
      }
      catch (Exception exp)
      {
        Log.Error(exp);
        return string.Empty;
      }
    }

    private static string GetPowerDVD9ExePath()
    {
      try
      {
        RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\CyberLink\PowerDVD9\");
        if (key == null)
          return string.Empty;

        object value = key.GetValue("InstallPath", null);
        if (value == null)
          return string.Empty;

        string path = value.ToString() + @"\PowerDVD9.exe";
        if (!System.IO.File.Exists(path))
          return string.Empty;

        return path;
      }
      catch (Exception exp)
      {
        Log.Error(exp);
        return string.Empty;
      }
    }
  }
}
