#region Copyright (C) 2005-2008 Team MediaPortal

/* 
 *	Copyright (C) 2005-2008 Team MediaPortal
 *	http://www.team-mediaportal.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#endregion

using System;
using System.Windows.Forms;

using MediaPortal.Configuration;

namespace WindowPlugins.BluRayPlayerLauncher
{
  public partial class PluginSetupForm : MediaPortal.UserInterface.Controls.MPConfigForm
  {
    public PluginSetupForm()
    {
      InitializeComponent();
      LoadSettings();
    }

    #region Serialisation

    private void LoadSettings()
    {
      using (MediaPortal.Profile.Settings xmlreader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MediaPortal.xml")))
      {
        Players player;
        string playerPath = xmlreader.GetValueAsString("BluRayPlayerLauncher", "executable", "...uMCEDVDPlayer.exe");

        int playerId = xmlreader.GetValueAsInt("BluRayPlayerLauncher", "player", -1);
        if (playerId < 0)
        {
          playerPath = InstalledPlayers.GetPlayerExePath(Players.Arcsoft);
          if (playerPath != string.Empty)
          {
            playerId = (int)Players.Arcsoft;
          }
          else
          {
            playerPath = InstalledPlayers.GetPlayerExePath(Players.Cyberlink);
            if (playerPath != string.Empty)
            {
              playerId = (int)Players.Cyberlink;
            }
          }
        }

        if (playerId < 0)
        {
          player = Players.Arcsoft;
        }
        else
        {
          player = (Players)playerId;
        }

        textBox_Filename.Text = playerPath;

        radioButton_Arcsoft.CheckedChanged -= new EventHandler(radioButton_Arcsoft_CheckedChanged);
        radioButton_Cyberlink.CheckedChanged -= new EventHandler(radioButton_Cyberlink_CheckedChanged);

        switch (player)
        {
          case Players.Arcsoft:
            radioButton_Arcsoft.Checked = true;
            break;
          case Players.Cyberlink:
            radioButton_Cyberlink.Checked = true;
            break;
        }

        radioButton_Arcsoft.CheckedChanged += new EventHandler(radioButton_Arcsoft_CheckedChanged);
        radioButton_Cyberlink.CheckedChanged += new EventHandler(radioButton_Cyberlink_CheckedChanged);

        checkBox_Minimize.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "minimizeDuringPlay", true);
        checkBox_SuspendRendering.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "suspendRenderingDuringPlay", true);

        checkBox_AutoplayHDDVD.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "autoplayHDDVD", true);
        checkBox_AutoplayBluray.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "autoplayBluray", true);
        checkBox_AutoplayASK.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "autoplayASK", true);

        checkBox_ChangeRefreshrate.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "changeRefreshRate", true);
        checkBox_24hzRefreshRate.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "24hzRefreshRate", false);

        checkBox_RestorePowerState.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "restorePowerState", false);
        checkBox_ShowInMenu.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "showInMenu", true);
        checkBox_KeyboardTranslation.Checked = xmlreader.GetValueAsBool("BluRayPlayerLauncher", "KeyboardTranslation", true);
        textBox_PluginNameInMenu.Text = xmlreader.GetValueAsString("BluRayPlayerLauncher", "pluginNameInMenu", "");

      } //using (MediaPortal.Profile.Settings xmlreader
    }

    void SaveSettings()
    {
      using (MediaPortal.Profile.Settings xmlwriter = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MediaPortal.xml")))
      {
        Players player = Players.Arcsoft;

        if (radioButton_Cyberlink.Checked)
          player = Players.Cyberlink;

        xmlwriter.SetValue("BluRayPlayerLauncher", "player", (int)player);

        xmlwriter.SetValue("BluRayPlayerLauncher", "pluginNameInMenu", textBox_PluginNameInMenu.Text);
        xmlwriter.SetValue("BluRayPlayerLauncher", "executable", textBox_Filename.Text);

        if (checkBox_Minimize.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "minimizeDuringPlay", "yes");
        }
        else 
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "minimizeDuringPlay", "no");
        }

        if (checkBox_SuspendRendering.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "suspendRenderingDuringPlay", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "suspendRenderingDuringPlay", "no");
        }

        if (checkBox_AutoplayHDDVD.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayHDDVD", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayHDDVD", "no");
        }

        if (checkBox_AutoplayBluray.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayBluray", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayBluray", "no");
        }

        if (checkBox_AutoplayASK.Checked == true)
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayASK", "yes");
        }
        else
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "autoplayASK", "no");
        }

        if (checkBox_ChangeRefreshrate.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "changeRefreshRate", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "changeRefreshRate", "no");
        }

        if (checkBox_24hzRefreshRate.Checked == true)
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "24hzRefreshRate", "yes");
        }
        else
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "24hzRefreshRate", "no");
        }

        if (checkBox_ShowInMenu.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "showInMenu", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "showInMenu", "no");
        }

        if (checkBox_KeyboardTranslation.Checked == true)
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "KeyboardTranslation", "yes");
        }
        else
        {
            xmlwriter.SetValue("BluRayPlayerLauncher", "KeyboardTranslation", "no");
        }

        if (checkBox_RestorePowerState.Checked == true)
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "restorePowerState", "yes");
        }
        else
        {
          xmlwriter.SetValue("BluRayPlayerLauncher", "restorePowerState", "no");
        }

      }
    }

    #endregion


    private void PluginSetupForm_Load(object sender, EventArgs e)
    {
    }

    private void button_OK_Click(object sender, EventArgs e)
    {
      if (!System.IO.File.Exists(textBox_Filename.Text))
      {
        System.Windows.Forms.MessageBox.Show("Invalid executable specified!", "Error");
        return;
      }

      SaveSettings();
      this.Close();
    }

    private void button_cancel_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void button_browse_Click(object sender, EventArgs e)
    {
      if (System.IO.File.Exists(textBox_Filename.Text))
      {
        openFileDialog1.FileName = System.IO.Path.GetFileName(textBox_Filename.Text);
      }
      else
      {
        if (radioButton_Arcsoft.Checked)
          openFileDialog1.FileName = "uMCEDVDPlayer.exe";
        else if (radioButton_Cyberlink.Checked)
          openFileDialog1.FileName = "PowerDVD.exe";
      }

      openFileDialog1.Filter.EndsWith(".exe");
      if (textBox_Filename.Text.Length >= 3) // prevents exception with wrong paths
      {
          openFileDialog1.InitialDirectory = System.IO.Path.GetDirectoryName(textBox_Filename.Text);
      }
      openFileDialog1.ShowDialog();
      textBox_Filename.Text = openFileDialog1.FileName;
    }

    private void radioButton_Cyberlink_CheckedChanged(object sender, EventArgs e)
    {
      if (!radioButton_Cyberlink.Checked)
        return;

      textBox_Filename.Text = InstalledPlayers.GetPlayerExePath(Players.Cyberlink);
    }

    private void radioButton_Arcsoft_CheckedChanged(object sender, EventArgs e)
    {
      if (!radioButton_Arcsoft.Checked)
        return;

      textBox_Filename.Text = InstalledPlayers.GetPlayerExePath(Players.Arcsoft);
    }

    private void checkBox1_CheckedChanged(object sender, EventArgs e)
    {

    }

    private void label2_Click(object sender, EventArgs e)
    {

    }

    private void checkBox1_CheckedChanged_1(object sender, EventArgs e)
    {

    }

    private void checkBox_AutoplayHDDVD_CheckedChanged(object sender, EventArgs e)
    {

    }

  }
}