using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

using MediaPortal.GUI.Library;

namespace WindowPlugins.BluRayPlayerLauncher
{
  internal static class RemoteControl
  {
    [DllImport("user32.dll")]
    static extern int GetForegroundWindow();

    [DllImport("user32.dll")]
    static extern int SetForegroundWindow(int hwnd);

    [DllImport("user32.dll")]
    static extern int SetFocus(int hwnd);

    [DllImport("user32.dll")]
    static extern int GetWindowThreadProcessId(int hwnd, int lpdwProcessId);

    [DllImport("user32.dll")]
    static extern int AttachThreadInput(int idAttach, int idAttachTo, bool fAttach);

    [DllImport("user32.dll")]
    static extern int ShowWindow(int hwnd, int nCmdShow);

    const int SW_MAXIMIZE = 3;

    /// <summary>
    /// Translates a MediaPortal action to the matching player keyboard combo
    /// </summary>
    public static void ProcessAction(Players player, Process playerProcess, MediaPortal.GUI.Library.Action.ActionType action)
    {
      switch (action)
      {
        case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREVIOUS_MENU:

          RemoteControl.ClosePlayer(playerProcess);

          break;
        default:

          string keys = RemoteControl.TranslateActionToPlayerKeyCombo(player, action);
          if (keys != string.Empty)
          {
            RemoteControl.SendKeyComboToPlayer(playerProcess, keys);
          }

          break;
      }
    }

    /// <summary>
    /// Translates a MediaPortal action to the matching player keyboard combo
    /// </summary>
    private static string TranslateActionToPlayerKeyCombo(Players player, MediaPortal.GUI.Library.Action.ActionType action)
    {
      switch (player)
      {
        case Players.Arcsoft:

          switch (action)
          {
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PLAY:
              return "{ENTER}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PAUSE:
              return " ";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_STOP:
              return "o";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_REWIND:
              return "r";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_FORWARD:
              return "f";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_ITEM:
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_CHAPTER:
              return "{PGDN}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREV_ITEM:
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREV_CHAPTER:
              return "{PGUP}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_SELECT_ITEM:
              return "{ENTER}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_UP:
              return "{UP}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_DOWN:
              return "{DOWN}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_LEFT:
              return "{LEFT}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_RIGHT:
              return "{RIGHT}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_DVD_MENU:
              return "^M";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_AUDIO:
              return "a"; // was: "l", why ? it is "a"
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_SUBTITLE:
              return "s";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_CONTEXT_MENU:
              return "^U";
          }

          break;

        case Players.Cyberlink:

          switch (action)
          {
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PLAY:
              return "{ENTER}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PAUSE:
              return " ";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_STOP:
              return "s";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_REWIND:
              return "b";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_FORWARD:
              return "f";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_ITEM:
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_CHAPTER:
              return "n";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREV_ITEM:
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_PREV_CHAPTER:
              return "p";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_SELECT_ITEM:
              return "{ENTER}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_UP:
              return "{UP}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_DOWN:
              return "{DOWN}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_LEFT:
              return "{LEFT}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_MOVE_RIGHT:
              return "{RIGHT}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_DVD_MENU:
              return "l";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_AUDIO:
              return "h";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_NEXT_SUBTITLE:
              return "u";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_SHOW_FULLSCREEN:
              return "{F5}";
            case MediaPortal.GUI.Library.Action.ActionType.ACTION_SHOW_PLAYLIST:
              return "^O";
          }

          break;
      }

      return string.Empty;
    }

    /// <summary>
    /// Sends keys via SendKeys.SendWait to the player and puts the window to foreground, if needed.
    /// </summary>
    private static void SendKeyComboToPlayer(Process playerProcess, string keys)
    {
      if (playerProcess == null)
      {
        Log.Error("playerProcess == null");
        return;
      }

      if (playerProcess.MainWindowHandle == IntPtr.Zero)
      {
        Log.Error("playerProcess.MainWindowHandle == IntPtr.Zero");
        return;
      }

      int foregroundWnd = GetForegroundWindow();
      int playerWnd = playerProcess.MainWindowHandle.ToInt32();

      if (foregroundWnd != playerWnd)
      {
        int threadIdForegroundWnd = GetWindowThreadProcessId(foregroundWnd, 0);
        int threadIdPlayerWnd = GetWindowThreadProcessId(playerWnd, 0);

        //cause we are controlling an external application, it is always running in another thread
        //To make sure that we get it in foreground we have to use AttachThreadInput

        if (threadIdForegroundWnd != threadIdPlayerWnd)
        {
          AttachThreadInput(threadIdForegroundWnd, threadIdPlayerWnd, true);

          SetForegroundWindow(playerWnd);
          SetFocus(playerWnd);

          AttachThreadInput(threadIdForegroundWnd, threadIdPlayerWnd, false);
        }
        else
        {
          SetForegroundWindow(playerWnd);
          SetFocus(playerWnd);
        }
      }

      SendKeys.SendWait(keys);
    }

    /// <summary>
    /// Maximizes the Player window with ShowWindow.
    /// </summary>
    private static void MaximizePlayerWindow(Process playerProcess)
    {
      if (playerProcess == null)
      {
        Log.Error("playerProcess == null");
        return;
      }

      if (playerProcess.MainWindowHandle == IntPtr.Zero)
      {
        Log.Error("playerProcess.MainWindowHandle == IntPtr.Zero");
        return;
      }

      ShowWindow(playerProcess.MainWindowHandle.ToInt32(), SW_MAXIMIZE);
    }

    /// <summary>
    /// Closes the Player.
    /// </summary>
    private static void ClosePlayer(Process playerProcess)
    {
      playerProcess.Kill();

      if (playerProcess == null)
      {
        Log.Error("playerProcess == null");
        return;
      }
        playerProcess.Kill();
    }
  }
}
